//
// LGDBasicPortProtocol.h
// LGDCommunicationKit, Release 1.0 (prerelease)
//
// Copyright (c) 1993, Looking Glass Design, Inc.
// All rights reserved.
//
// Project Manager:  Paul Girone
// Original Author:  Mike Gobbi
// Creation Date:    June 15, 1993
// $Revision: 1.3 $
//

// Disciplines
// (Referenced definitions found in <sgtty.h>)
#define LGD_SP_NEW	NTTYDISC
#define LGD_SP_OLD	OTTYDISC

// Modes
#define LGD_SP_RAW		0x0000
#define LGD_SP_CBREAK		0x0001
#define LGD_SP_COOKED		0x0002

// Speeds
// (Referenced definitions found in <sgtty.h>)
#define LGD_SP_B0	B0
#define LGD_SP_B50	B50
#define LGD_SP_B75	B75
#define LGD_SP_B110	B110
#define LGD_SP_B134	B134
#define LGD_SP_B150	B150
#define LGD_SP_B200	B200
#define LGD_SP_B300	B300
#define LGD_SP_B600	B600
#define LGD_SP_B1200	B1200
#define LGD_SP_B1800	B1800
#define LGD_SP_B2400	B2400
#define LGD_SP_B4800	B4800
#define LGD_SP_B9600	B9600

// Parity
// (Referenced definitions found in <sgtty.h>)
#define LGD_SP_EVENPARITY	EVENP
#define LGD_SP_ODDPARITY	ODDP
#define LGD_SP_ZEROPARITY	0x0000
#define LGD_SP_NOPARITY		(EVENP|ODDP)

// Duplex
#define LGD_SP_HALFDUPLEX	0x0000
#define LGD_SP_FULLDUPLEX	0x0001

@protocol LGDBasicPortProtocol

// Connecting
- (BOOL)isDeviceAvailable:(const char *)device;
- openAndLockDevice:(const char *)device mode:(int)mode timeout:(long)seconds;
- unlockAndClose;
- (BOOL)isOpen;

// Configuring
- setDiscipline:(int)aDiscipline;
- setBaudRate:(int)rate;
- setParity:(int)parity;
- setCRMassage:(BOOL)yn;
- setDuplex:(int)duplex;
- setMode:(int)mode;
- (int)discipline;
- (int)baudRate;
- (int)parity;
- (BOOL)CRMassage;
- (int)mode;

// Signals
- raiseBreak;
- lowerBreak;
- raiseDTR;
- lowerDTR;
- pause;
- continue;

// Communication
- (BOOL)dataAvailable;
- (int)sendBlock:(const char *)buffer size:(int)size all:(BOOL)yn;
- (int)getBlock:(char *)buffer size:(int)bufSize all:(BOOL)yn;

// Buffers
- flushInput;
- flushOutput;

@end
