/*  
** Copyright (c) 1995 Netsurfer Inc.  All Rights Reserved.
**
** Author: <bbum@friday.com>
*/

/*  This object is included in the MiscKit by permission from the author
**  and its use is governed by the MiscKit license, found in the file
**  "LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
**  for a list of all applicable permissions and restrictions.
*/

@interface SurfDecoder : Object
{
	struct __SurfDecodeFlags {
		unsigned char	verboseMode:1; /*" YES if verbosity enabled "*/
		unsigned char	lastCorrupt:1; /*" YES if last image corrupt "*/
	} _SDFlags;

	id					returnImage; /*"image eventually returned to caller "*/
	NXWindowDepth		imageDepth; /*" target decoding depth "*/

	id <SurfErrorDelegate> 	errorDelegate;		/*"Error Delegate"*/
}

/*"
 *  Shared Instance
"*/
+ sharedInstance;

/*"
 * Filterable Type Query
"*/
+ (const char *const *)imageUnfilteredFileTypes;
+ (BOOL)canLoadFromStream:(NXStream *)stream;

/*"
 * Designated Initializer
"*/
- init;

/*"
 * Decoding an Image
"*/
- _decodeFromStream:(NXStream *) theStream;

- decodeFromFile:(const char *) filePath;
- decodeFromFile:(const char *) filePath intoImage: image;

- decodeFromStream:(NXStream *) stream;
- decodeFromStream:(NXStream *) stream intoImage:image;

/*"
 *  Set / Query
"*/
- (void) setImageDepth:(NXWindowDepth) aDepth;
- (NXWindowDepth) imageDepth;

- (void) setLastImageCorrupt:(BOOL) aFlag; // INTERNAL ONLY!
- (BOOL) lastImageCorrupt;

- (BOOL) verboseMode;
- (void) setVerboseMode:(BOOL) aFlag;

- (void) setErrorDelegate:(id <SurfErrorDelegate>) aDelegate;
- errorDelegate;

/*"
 *  Error Handling
"*/
- (void) spewMessage:(const char *) errMsg
		withSeverity:(SurfErrorSeverity) aSeverity;
@end
