/* Interface for Objective-C Collection object

   Copyright (C) 1993 R. Andrew McCallum <mccallum@cs.rochester.edu>
   Dept. of Computer Science, U. of Rochester, Rochester, NY  14627

   This file is part of the GNU Objective-C Collection library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/* This is the abstract superclass that satisfies the Collecting
   protocol.  
*/

#ifndef __Collection_h_INCLUDE_GNU
#define __Collection_h_INCLUDE_GNU

#include <objc/Object.h>
#include <objc/hash.h>
#include <coll/Collecting.h>
#include <coll/collstd.h>

@interface Collection : Object <Collecting>
{
    char *_description;
    compare_func_type _compare_func;
}

// WARNING;
- warning:(const char *)format,...;
/* We should have a more general mechanism for things like this. */

+ printElement: (elt)anElement description: (const char *)type;
- printElement: (elt)anElement;
- printForDebugger;

@end

/* The only subclassResponsibilities in Collection are:

      addElement:
      removeElement:
      getNextElement:withEnumState:

   but subclass will want to override others as well in order to 
   increase efficiency, especially:

      count
      empty
      includesElement:
      occurrencesOfElement:

   and perhaps:

      uniqueContents
      isEmpty
      withElementsCall:whileTrue:
      withElementsCall:

*/

#endif /* __Collection_h_INCLUDE_GNU */

