/* Protocol for Objective-C objects that use a comparitor while holding
   elements in a sequence.

   Copyright (C) 1993 R. Andrew McCallum <mccallum@cs.rochester.edu>
   Dept. of Computer Science, U. of Rochester, Rochester, NY  14627

   This file is part of the GNU Objective-C Collection library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/* The <ComparedCollecting> protocol inherits from the 
   <IndexedCollecting> protocol.

   The <ComparedCollecting> protocol defines the interface to a
   collection of elements whose sequence is determined by some comparison
   operator.
*/

#ifndef __ComparedCollecting_h_INCLUDE_GNU
#define __ComparedCollecting_h_INCLUDE_GNU

#include <coll/IndexedCollecting.h>

@protocol ComparedCollecting <IndexedCollecting>

// INITIALIZING;
- initComparisonSel: (SEL)aCompareSel;

// MANAGING COMPARATOR;
- setComparisonSel: (SEL)aCompareSel;
- (SEL) comparisonSel;
- (BOOL) comparatorIsSel;

// NON-OBJECT ELEMENT METHOD NAMES;
- initDescription: (const char *)type 
    comparisonFunc: (int(*)(elt,elt))aFunc;
- initComparisonFunc: (int(*)(elt,elt))aFunc;
- setComparisonFunc: (int(*)(elt,elt))aFunc;
- (int(*)(elt,elt)) comparisonFunc;
- (BOOL) comparatorIsFunc;

@end

#endif /* __ComparedCollecting_h_INCLUDE_GNU */

