/* Support for general purpose definitions for the Collection Library.

   Copyright (C) 1993 R. Andrew McCallum <mccallum@cs.rochester.edu>
   Dept. of Computer Science, U. of Rochester, Rochester, NY  14627

   This file is part of the GNU Objective-C Collection library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include <stdlib.h>
#include <objc/Object.h>
#include <objc/hash.h>

/* defined in objc/misc.c */
extern void * __objc_xrealloc (void *optr, size_t size);
extern void * __objc_xmalloc (size_t size);
extern void * __objc_xcalloc(size_t nelem, size_t size);

/* I do this to make substituting Boehm's GC easy. */
void *(*objc_malloc)(size_t size) = __objc_xmalloc;
void *(*objc_realloc)(void *optr, size_t size) = __objc_xrealloc;
void *(*objc_calloc)(size_t nelem, size_t size) = __objc_xcalloc;
void (*objc_free)(void *optr) = free;

@implementation Object (shouldNotImplement)
- shouldNotImplement: (SEL)aSel
{
  return [self error:"should not implement %s", sel_get_name(aSel)];
}
@end

/* Eventually I'd like this to be added to hash.c */
node_ptr 
hash_node_for_key (cache_ptr cache, const void *key)
{
  node_ptr node = cache->node_table[(*cache->hash_func)(cache, key)];

  if (node)
    do {
      if ((*cache->compare_func)(node->key, key))
	return node;
      else
        node = node->next;
    } while (node);

  return 0;
}
  

