/* Implementation for Objective-C LinkedListNode object
   Copyright (C) 1993,1994 Free Software Foundation, Inc.

   Written by:  R. Andrew McCallum <mccallum@cs.rochester.edu>
   Dept. of Computer Science, U. of Rochester, Rochester, NY  14627

   This file is part of the GNU Objective-C Collection library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include <coll/LinkedListNode.h>

@implementation LinkedListNode

+ initialize
{
  if (self == [LinkedListNode class])
    [self setVersion:-1];	/* alpha release */
  return self;
}

- init
{
  [super init];
  _next = _prev = nil;
  return self;
}

- write: (TypedStream*)aStream
{
  [super write:aStream];
  objc_write_object_reference(aStream, _next);
  objc_write_object_reference(aStream, _prev);
  return self;
}

- read: (TypedStream*)aStream
{
  [super read:aStream];
  objc_read_object(aStream, &_next);
  objc_read_object(aStream, &_prev);
  return self;
}

- (id <LinkedListComprising>) nextLink
{
  return _next;
}

- (id <LinkedListComprising>) prevLink
{
  return _prev;
}

- setNextLink: (id <LinkedListComprising>)aLink
{
  _next = aLink;
  return self;
}

- setPrevLink: (id <LinkedListComprising>)aLink
{
  _prev = aLink;
  return self;
}

@end

