all::

OBJCFLAGS=-Wno-import

SUPPORTFILES = Makefile makefile \
	Makefile.preamble Makefile.postamble SGS_ENV

INSTALLHDRS = $(CLASSES:.m=.h) $(HFILES)

SRCFILES = $(CLASSES) $(CLASSES:.m=.h) \
	$(MFILES) $(CFILES) $(HFILES) $(PSWFILES) $(PSWMFILES) \
	$(GLOBAL_RESOURCES) \
	$(OTHERSRCS) $(OTHERLINKED) $(OTHER_SOURCEFILES)

EARLY_OFILES = $(PSWFILES:.psw=.o) $(PSWMFILES:.pswm=.o)

INITIAL_TARGETS = $(OFILE_DIR) $(EARLY_OFILES) $(PRECOMPS) $(OTHER_INITIAL_TARGETS)

OFILES = $(CLASSES:.m=.o) $(MFILES:.m=.o) $(CFILES:.c=.o) \
	$(EARLY_OFILES) $(OTHERLINKEDOFILES)

MAKEFILES = Makefile

PRODUCT_DEPENDS = $(OFILES) $(OTHER_OFILES) $(MAKEFILES) \
	$(OTHER_PRODUCT_DEPENDS) $(YAMFILES:.yam=.men)

GARBAGE = $(PSWFILES:.psw=.c) $(PSWMFILES:.pswm=.m) \
	$(PSWFILES:.psw=.h) $(PSWMFILES:.pswm=.m) \
	$(YAMFILES:.yam=.men) \
	$(NAME) $(NAME).debug $(NAME).profile \
	lib$(NAME).a lib$(NAME)_g.a lib$(NAME)_p.a \
	obj debug_obj profile_obj $(OTHER_GARBAGE) gmon.out \
	$(PRECOMPS) $(OTHER_INITIAL_TARGETS)

REAL_GARBAGE = $(GARBAGE) *~ Makefile.dependencies tags TAGS

# Defaults for who to chown executables to when installing
INSTALL_AS_USER = root
INSTALL_AS_GROUP = wheel

# Defaults for options to $(STRIP)
RELOCATABLE_STRIP_OPTS = -x -u
DYLD_APP_STRIP_OPTS = -A -n
APP_STRIP_OPTS =   # Set this to $(DYLD_APP_STRIP_OPTS) when doing dynaloading yourself

# Various commands:
SHELL  = /bin/sh
FASTCP = /usr/lib/fastcp
CHANGES = /usr/lib/changes
CHOWN  = /etc/chown
CHMOD  = /bin/chmod
TAR    = /bin/tar
STRIP  = /bin/strip
RM     = /bin/rm
ECHO   = /bin/echo
MKDIRS = /bin/mkdirs
TOUCH  = /usr/bin/touch
AWK    = /bin/awk
AR     = /bin/ar
RANLIB = /bin/ranlib
YACC   = /usr/bin/yacc
LEX    = /usr/bin/lex
DOYAM  = /usr/local/bin/doyam
CP     = /bin/cp
GREP   = /bin/grep
CVS    = /usr/local/bin/cvs

VPATH = $(OFILE_DIR):$(PRODUCT_ROOT)

# Change this for if you want `pwd` or something similar.
srcdir=`pwd`

.SUFFIXES: .h .m .c .psw .pswm .o .p .clean .l .y .yam .men

copy::
	@($(MAKE) subcopy \
		"SRCDIR=$(srcdir)" \
		"PRODUCT_ROOT = ." \
		"PRODUCT = $(NAME).copy" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)

clean::
	@($(MAKE) subclean \
		"SRCDIR=$(srcdir)" \
		"PRODUCT_ROOT = ." \
		"PRODUCT = $(NAME).clean" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)

realclean::
	@($(MAKE) subrealclean \
		"SRCDIR=$(srcdir)" \
		"PRODUCT_ROOT = ." \
		"PRODUCT = $(NAME).realclean" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)

depend::
	@($(MAKE) subdepend \
		"SRCDIR=$(srcdir)" \
		"PRODUCT_ROOT = ." \
		"PRODUCT = $(NAME).depend" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)

install::
	@($(MAKE) subinstall \
		"SRCDIR=$(srcdir)" \
		"PRODUCT_ROOT = ." \
		"PRODUCT = $(NAME).install" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)

installsrc::
	@($(MAKE) subinstallsrc \
		"SRCDIR=$(srcdir)" \
		"PRODUCT_ROOT = ." \
		"PRODUCT = $(NAME).installsrc" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)

ctags::
	@($(MAKE) subctags \
		"SRCDIR=$(srcdir)" \
		"PRODUCT_ROOT = ." \
		"PRODUCT = $(NAME).ctags" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)

etags::
	@($(MAKE) subetags \
		"SRCDIR=$(srcdir)" \
		"PRODUCT_ROOT = ." \
		"PRODUCT = $(NAME).etags" \
		"TMMAKEFILEDIR = $(TMMAKEFILEDIR)" ; \
	)

# XXX HACK
SYM_DIR=.

# Compilation rules:
.c.o:
	$(CC) $(CFLAGS) -c $*.c -o $(OFILE_DIR)/$*.o
.m.o:
	$(CC) $(CFLAGS) $(OBJCFLAGS) -c $*.m -o $(OFILE_DIR)/$*.o
.h.p:
	$(CC) -precomp $(CFLAGS) $*.h -o $*.p

# yacc and lex-related rules
.y.o:
        @(initdir=`pwd`;                                                \
        cd $(SYM_DIR);                                                  \
          cmd="$(YACC) $(YFLAGS) $$initdir/$*.y" ; echo $$cmd; $$cmd ;  \
          cmd="$(MV) $(MVFLAGS) y.tab.c $*.c" ; echo $$cmd; $$cmd ;     \
          cmd="$(CP) $(MVFLAGS) y.tab.h $*.h" ; echo $$cmd; $$cmd ;     \
        cd $$initdir ;                                                  \
        cmd="$(CC) $(ALL_CFLAGS) -I$$initdir -c $(SYM_DIR)/$*.c -o $(OFILE_DIR)/$*.o" ; \
        echo $$cmd ; $$cmd )

.y.c .y.h:
        @(initdir=`pwd`;                                                \
        cd $(SYM_DIR);                                                  \
          cmd="$(YACC) $(YFLAGS) $$initdir/$*.y" ; echo $$cmd; $$cmd ;  \
          cmd="$(MV) $(MVFLAGS) y.tab.c $*.c" ; echo $$cmd; $$cmd ;     \
          cmd="$(CP) $(MVFLAGS) y.tab.h $*.h" ; echo $$cmd; $$cmd )

.ym.o:
        @(initdir=`pwd`;                                                 \
        cd $(SYM_DIR);                                                  \
          cmd="$(YACC) $(YFLAGS) $$initdir/$*.ym" ; echo $$cmd; $$cmd ; \
          cmd="$(MV) $(MVFLAGS) y.tab.c $*.m" ; echo $$cmd; $$cmd ;     \
          cmd="$(CP) $(MVFLAGS) y.tab.h $*.h" ; echo $$cmd; $$cmd ;     \
        cd $$initdir ;                                                  \
        cmd="$(CC) $(ALL_CFLAGS) -I$$initdir -c $(SYM_DIR)/$*.m -o $(OFILE_DIR)/$*.o" ; \
        echo $$cmd ; $$cmd )

.ym.m .ym.h:
        @(initdir=`pwd`;                                                 \
        cd $(SYM_DIR);                                                  \
          cmd="$(YACC) $(YFLAGS) $$initdir/$*.ym" ; echo $$cmd; $$cmd ; \
          cmd="$(MV) $(MVFLAGS) y.tab.c $*.m" ; echo $$cmd; $$cmd ;     \
          cmd="$(CP) $(MVFLAGS) y.tab.h $*.h" ; echo $$cmd; $$cmd )

.l.o:
        @(initdir=`pwd`;                                                \
        cd $(SYM_DIR);                                                  \
          cmd="$(LEX) $(LFLAGS) $$initdir/$*.l" ; echo $$cmd; $$cmd ;   \
          cmd="$(MV) $(MVFLAGS) lex.yy.c $*.c" ; echo $$cmd; $$cmd ;    \
        cd $$initdir ;                                                  \
        cmd="$(CC) $(ALL_CFLAGS) -I$$initdir -c $(SYM_DIR)/$*.c -o $(OFILE_DIR)/$*.o" ; \
        echo $$cmd ; $$cmd )

.l.c:
        @(initdir=`pwd`;                                                \
        cd $(SYM_DIR);                                                  \
          cmd="$(LEX) $(LFLAGS) $$initdir/$*.l" ; echo $$cmd; $$cmd ;   \
          cmd="$(MV) $(MVFLAGS) lex.yy.c $*.c" ; echo $$cmd; $$cmd )

.lm.o:
        @(initdir=`pwd`;                                                \
        cd $(SYM_DIR);                                                  \
          cmd="$(LEX) $(LFLAGS) $$initdir/$*.lm" ; echo $$cmd; $$cmd ;  \
          cmd="$(MV) $(MVFLAGS) lex.yy.c $*.m" ; echo $$cmd; $$cmd ;    \
        cd $$initdir ;                                                  \
        cmd="$(CC) $(ALL_CFLAGS) -I$$initdir -c $(SYM_DIR)/$*.m -o $(OFILE_DIR)/$*.o" ; \
        echo $$cmd ; $$cmd )

.lm.m:
        @(initdir=`pwd`;                                                \
        cd $(SYM_DIR);                                                  \
          cmd="$(LEX) $(LFLAGS) $$initdir/$*.lm" ; echo $$cmd; $$cmd ;  \
          cmd="$(MV) $(MVFLAGS) lex.yy.c $*.m" ; echo $$cmd; $$cmd )


# pswrap-related rules:
.psw.h:
	$(PSWRAP) $(PSWFLAGS) -a -h $*.h -o $*.c $*.psw
.psw.c:
	$(PSWRAP) $(PSWFLAGS) -a -h $*.h -o $*.c $*.psw
.psw.o:
	$(PSWRAP) $(PSWFLAGS) -a -h $*.h -o $*.c $*.psw
	$(CC) $(CFLAGS) -c $*.c -o $(OFILE_DIR)/$*.o
.pswm.h:
	$(PSWRAP) $(PSWFLAGS) -a -h $*.h -o $*.m $*.pswm
.pswm.m:
	$(PSWRAP) $(PSWFLAGS) -a -h $*.h -o $*.m $*.pswm
.pswm.o:
	$(PSWRAP) $(PSWFLAGS) -a -h $*.h -o $*.m $*.pswm
	$(CC) $(CFLAGS) $(OBJCFLAGS) -c $*.m -o $(OFILE_DIR)/$*.o

# Menu releated rules:
.yam.men:
	$(DOYAM) $(YAMFLAGS) -o $*.men $*.yam

SRCROOT DEST:
	@if [ -n "${$@}" ]; then exit 0; \
	else $(ECHO) Must define $@; exit 1; fi

$(DEST)::
	-$(RM) -rf $(DEST)
	-$(MKDIRS) $(DEST)

subcopy:: DEST $(DEST) subcomponents
	@if [ "$(SRCFILES)" != "" ] ; then \
	       $(ECHO) "$(TAR) cf - $(SRCFILES) | (cd $(DEST); $(TAR) xf -)" ; \
	       $(TAR) cf - $(SRCFILES) | (cd $(DEST); $(TAR) xf -) ; \
	fi
	@supportfiles=""; \
	for i in $(SUPPORTFILES) none ; do \
	    if [ -r $$i -a ! -r $(DEST)/$$i ] ; then \
		supportfiles="$$supportfiles $$i" ; \
	    fi ; \
	done ; \
	if [ "$$supportfiles" != "" ] ; then \
	   $(ECHO) "$(TAR) cf - $$supportfiles | (cd $(DEST); $(TAR) xf -)" ; \
	   $(TAR) cf - $$supportfiles | (cd $(DEST); $(TAR) xf -) ; \
	fi

subclean:: subcomponents
	$(RM) -rf $(GARBAGE)

subrealclean:: subcomponents
	$(RM) -rf $(REAL_GARBAGE)

# Build a set of dependencies for current level into Makefile.dependencies
subdepend:: $(CLASSES) $(CLASSES:.m=.h) $(MFILES) $(CFILES) $(HFILES) subcomponents
	$(RM) -f Makefile.dependencies
	$(CC) -M $(CFLAGS) $(OTHER_CFLAGS) $(CLASSES) $(MFILES) $(CFILES) | \
	    $(AWK) '{ if ($$1 != prev) { if (rec != "") print rec; \
		rec = $$0; prev = $$1; } \
		else { if (length(rec $$2) > 78) { print rec; rec = $$0; } \
		else rec = rec " " $$2 } } \
		END { print rec }' > Makefile.dependencies

subinstallsrc:: SRCROOT subcomponents
	@($(MAKE) copy "DEST=$(SRCROOT)")

subctags:: $(CLASSES) $(CLASSES:.m=.h) $(MFILES) $(CFILES) $(HFILES) subcomponents
	ctags $(CLASSES) $(CLASSES:.m=.h) $(MFILES) $(CFILES) $(HFILES) > tags

subetags:: $(CLASSES) $(CLASSES:.m=.h) $(MFILES) $(CFILES) $(HFILES) subcomponents
	etags $(CLASSES) $(CLASSES:.m=.h) $(MFILES) $(CFILES) $(HFILES) > TAGS

# The following rule does the recursion into any subprojects or bundles of this (sub)project and does a 'make project' for each one in its respective directories.  This insures that we do not rely on the directory timestamp or "hacK" file to know whether or not something has changed.

subcomponents::
	@(for sub in $(SUBPROJECTS) none ; do \
	   if [ $$sub = "none" ] ; then break; fi ;\
	   if [ -d $(SRCDIR)/$$sub ]; then \
	       echo make[1]: Entering directory \`$(SRCDIR)/$$sub\'; \
	       cd $(SRCDIR)/$$sub ; \
	       echo "$(MAKE) \"CC = $(CC)\" \"SRCDIR = $(SRCDIR)/$$sub\" \"TMMAKEFILEDIR = $(TMMAKEFILEDIR)\" `echo $(PRODUCT) | $(AWK) -F. '{print $$2}'`" ; \
	       $(MAKE) "SRCDIR = $(SRCDIR)/$$sub" "TMMAKEFILEDIR = $(TMMAKEFILEDIR)" `echo $(PRODUCT) | $(AWK) -F. '{print $$2}'` ; \
	       echo make[1]: Leaving directory \`$(SRCDIR)/$$sub\'; \
	       cd .. ; \
	   fi ; \
	done)

#subproject:: $(PRODUCT) subcomponents

$(OFILE_DIR):
	@$(MKDIRS) $(OFILE_DIR)
